#!/bin/sh
#
#	Compile an executable from a single .c file in the local
#	directory.  The .c file includes one local .h file
#	(double-quote #include) and one subdirectory .h file
#	(angle-bracke #include).  Specify "Ignore '^subdirectory/"
#	in the Construct file.  Build the executable by invoking
#	cons.  Update the subdirectory's .h files and invoke cons
#	again; check that it ignored the update .h file and did
#	NOT rebuild the executable.  Update the local .h file and
#	invoke cons again; check that it rebuilt the executable,
#	picking up the changes to both .h files.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [Ignore]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir include
if test $? -ne 0 ; then no_result ; fi

#
cat <<'_EOF_' >Construct
$env = new cons (
	CPPPATH => '.',
);
Ignore '^include/';
Program $env 'foo', 'foo.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >foo.c
#include "local.h"
#include <include/foo.h>
main()
{
	printf("This is the foo.c file:\n");
	printf("\tlocal.h version %d\n", LOCAL_VERSION);
	printf("\tfoo.h version %d\n", FOO_VERSION);
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

cat <<'_EOF_' >local.h
#define	LOCAL_VERSION	11
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >include/foo.h
#define	FOO_VERSION	21
_EOF_
if test $? -ne 0 ; then no_result ; fi
$prog . > cons.1.out 2>&1
if test $? -ne 0 ; then fail ; fi
./foo > foo.out
if test $? -ne 0 ; then fail ; fi
diff - foo.out <<'_EOF_'
This is the foo.c file:
	local.h version 11
	foo.h version 21
_EOF_
if test $? -ne 0 ; then fail ; fi

cat <<'_EOF_' >include/foo.h
#define	FOO_VERSION	22
_EOF_
if test $? -ne 0 ; then no_result ; fi
$prog . > cons.2.out 2>&1
if test $? -ne 0 ; then fail ; fi
./foo > foo.out
if test $? -ne 0 ; then fail ; fi
diff - foo.out <<'_EOF_'
This is the foo.c file:
	local.h version 11
	foo.h version 21
_EOF_
if test $? -ne 0 ; then fail ; fi

cat <<'_EOF_' >local.h
#define	LOCAL_VERSION	12
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >include/foo.h
#define	FOO_VERSION	23
_EOF_
$prog . > cons.3.out 2>&1
if test $? -ne 0 ; then fail ; fi
./foo > foo.out
if test $? -ne 0 ; then fail ; fi
diff - foo.out <<'_EOF_'
This is the foo.c file:
	local.h version 12
	foo.h version 23
_EOF_
if test $? -ne 0 ; then fail ; fi

#
pass
