#!/bin/sh
#
#	Create two repository subdirectories with a single .c and
#	Construct file in the "old" repository.  Invoke cons -R -R
#	in a work subdirectory.  Check that it built correctly.
#	Create a .c in the "new" repository; invoke cons -R -R;
#	check that the executable built with the new .c file.
#	Create a .c in the work subdirectory; cons -R -R; check
#	that the executable built with the work .c file.  Update
#	both repository .c files; cons -R -R; check that the
#	executable still uses the work .c file.  Remove the work
#	.c file; cons -R -R; check that the executable uses the
#	new repository .c file.  Remove the new repository .c file;
#	cons -R -R; check that the executable uses the old repository
#	.c file.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [single-module Program, multiple -R]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr, input: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir repository.NEW
if test $? -ne 0 ; then exit 2 ; fi
mkdir repository.OLD
if test $? -ne 0 ; then exit 2 ; fi
mkdir work
if test $? -ne 0 ; then exit 2 ; fi

#
cat <<'_EOF_' >repository.OLD/Construct	
$env = new cons;
Program $env 'foo', 'foo.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cp repository.OLD/Construct repository.NEW/construct
if test $? -ne 0 ; then no_result ; fi

cat <<'_EOF_' >repository.OLD/foo.c
main()
{
	printf("repository.OLD/foo.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

# Make the repositories non-writable,
# so we'll detect if we try to write into it accidentally.
find repository.NEW -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi
find repository.OLD -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository.NEW -R $work/repository.OLD .) >cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
diff - cons.1.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
work/foo > foo.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.1.out <<'_EOF_'
repository.OLD/foo.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

(set +x; cd work && $prog -R $work/repository.NEW -R $work/repository.OLD .) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 cons.2.out ; fi
diff - cons.2.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

chmod u+w repository.NEW
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository.NEW/foo.c
main()
{
	printf("repository.NEW/foo.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
chmod u-w repository.NEW/foo.c
if test $? -ne 0 ; then no_result ; fi
chmod u-w repository.NEW
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository.NEW -R $work/repository.OLD .) >cons.3.out 2>cons.3.err
if test $? -ne 0 ; then fail 3 cons.3.out ; fi
diff - cons.3.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 3 ; fi
work/foo > foo.3.out
if test $? -ne 0 ; then fail 3 ; fi
diff - foo.3.out <<'_EOF_'
repository.NEW/foo.c
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

(set +x; cd work && $prog -R $work/repository.NEW -R $work/repository.OLD .) >cons.4.out 2>cons.4.err
if test $? -ne 0 ; then fail 4 cons.4.out ; fi
diff - cons.4.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 4 ; fi
diff - cons.4.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 4 ; fi

cat <<'_EOF_' >work/foo.c
main()
{
	printf("work/foo.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository.NEW -R $work/repository.OLD .) >cons.5.out 2>cons.5.err
if test $? -ne 0 ; then fail 5 cons.5.out ; fi
diff - cons.5.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 5 ; fi
work/foo > foo.5.out
if test $? -ne 0 ; then fail 5 ; fi
diff - foo.5.out <<'_EOF_'
work/foo.c
_EOF_
if test $? -ne 0 ; then fail 5 ; fi

(set +x; cd work && $prog -R $work/repository.NEW -R $work/repository.OLD .) >cons.6.out 2>cons.6.err
if test $? -ne 0 ; then fail 6 cons.6.out ; fi
diff - cons.6.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 6 ; fi
diff - cons.6.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 6 ; fi

chmod u+w repository.NEW/foo.c
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository.NEW/foo.c
main()
{
	printf("repository.NEW/foo.c again\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
chmod u-w repository.NEW/foo.c
if test $? -ne 0 ; then no_result ; fi

chmod u+w repository.OLD/foo.c
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository.OLD/foo.c
main()
{
	printf("repository.OLD/foo.c again\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
chmod u-w repository.OLD/foo.c
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository.NEW -R $work/repository.OLD .) >cons.7.out 2>cons.7.err
if test $? -ne 0 ; then fail 7 cons.7.out ; fi
diff - cons.7.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 7 ; fi
work/foo > foo.7.out
if test $? -ne 0 ; then fail 7 ; fi
diff - foo.7.out <<'_EOF_'
work/foo.c
_EOF_
if test $? -ne 0 ; then fail 7 ; fi

(set +x; cd work && $prog -R $work/repository.NEW -R $work/repository.OLD .) >cons.8.out 2>cons.8.err
if test $? -ne 0 ; then fail 8 cons.8.out ; fi
diff - cons.8.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 8 ; fi
diff - cons.8.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 8 ; fi

rm -f work/foo.c
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository.NEW -R $work/repository.OLD .) >cons.9.out 2>cons.9.err
if test $? -ne 0 ; then fail 9 cons.9.out ; fi
diff - cons.9.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 9 ; fi
work/foo > foo.9.out
if test $? -ne 0 ; then fail 9 ; fi
diff - foo.9.out <<'_EOF_'
repository.NEW/foo.c again
_EOF_
if test $? -ne 0 ; then fail 9 ; fi

(set +x; cd work && $prog -R $work/repository.NEW -R $work/repository.OLD .) >cons.10.out 2>cons.10.err
if test $? -ne 0 ; then fail 10 cons.10.out ; fi
diff - cons.10.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 10 ; fi
diff - cons.10.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 10 ; fi

chmod u+w repository.NEW
if test $? -ne 0 ; then no_result ; fi
rm -f repository.NEW/foo.c
if test $? -ne 0 ; then no_result ; fi
chmod u-w repository.NEW
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository.NEW -R $work/repository.OLD .) >cons.11.out 2>cons.11.err
if test $? -ne 0 ; then fail 11 cons.11.out ; fi
diff - cons.11.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 11 ; fi
work/foo > foo.11.out
if test $? -ne 0 ; then fail 11 ; fi
diff - foo.11.out <<'_EOF_'
repository.OLD/foo.c again
_EOF_
if test $? -ne 0 ; then fail 11 ; fi

(set +x; cd work && $prog -R $work/repository.NEW -R $work/repository.OLD .) >cons.12.out 2>cons.12.err
if test $? -ne 0 ; then fail 12 cons.12.out ; fi
diff - cons.12.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 12 ; fi
diff - cons.12.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 12 ; fi

#
pass
