#!/bin/sh
#
#	Create src and include subdirectories in a work subdirectory
#	and a repository subdirectory.  CPPPATH in the repository
#	Construct file specifies src:include.  A .c file in the
#	src subdirectory #includes a .h file in the src subdirectory,
#	which nested #includes a .h file in the include subdirectory.
#	Build in the repository.  Build in the work directory;
#	everything should still be up-to-date.  Create a work copy
#	of the include subdirectory .h file; build.  Create a work
#	copy of the src subdirectory .h file; build.  Remove the
#	work copy of the include subdirectory .h file; build.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [-R, multi-dir .h]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir work/src
if test $? -ne 0 ; then no_result ; fi
mkdir work/include
if test $? -ne 0 ; then no_result ; fi

mkdir repository
if test $? -ne 0 ; then no_result ; fi
mkdir repository/src
if test $? -ne 0 ; then no_result ; fi
mkdir repository/include
if test $? -ne 0 ; then no_result ; fi

#
cat <<'_EOF_' >repository/Construct
$env = new cons(
	CPPPATH => 'src:include',
);
Export qw( env );
Build qw(
	src/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/Conscript
Import qw( env );
Program $env 'xxx', qw (
	main.c
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/include/my_string.h
#define	MY_STRING	"repository/include/my_string.h"
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/include.h
#include <my_string.h>
#define	LOCAL_STRING	"repository/src/include.h"
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/main.c
#include <include.h>
main()
{
	printf("%s\n", MY_STRING);
	printf("%s\n", LOCAL_STRING);
	printf("repository/src/main.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd repository; $prog .) > cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail 1 ; fi
repository/src/xxx > foo.1out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.1out <<'_EOF_'
repository/include/my_string.h
repository/src/include.h
repository/src/main.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 cons.2.out ; fi
diff - cons.2.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

cat <<'_EOF_' >work/include/my_string.h
#define	MY_STRING	"work/include/my_string.h"
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.3.out 2>cons.3.err
if test $? -ne 0 ; then fail 3 cons.3.out ; fi
work/src/xxx > foo.3.out
if test $? -ne 0 ; then fail 3 ; fi
diff - foo.3.out <<'_EOF_'
work/include/my_string.h
repository/src/include.h
repository/src/main.c
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

cat <<'_EOF_' >work/src/include.h
#include <my_string.h>
#define	LOCAL_STRING	"work/src/include.h"
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.4.out 2>cons.4.err
if test $? -ne 0 ; then fail 4 cons.4.out ; fi
work/src/xxx > foo.4.out
if test $? -ne 0 ; then fail 4 ; fi
diff - foo.4.out <<'_EOF_'
work/include/my_string.h
work/src/include.h
repository/src/main.c
_EOF_
if test $? -ne 0 ; then fail 4 ; fi

rm -f work/include/my_string.h
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.5.out 2>cons.5.err
if test $? -ne 0 ; then fail 5 cons.5.out ; fi
work/src/xxx > foo.5.out
if test $? -ne 0 ; then fail 5 ; fi
diff - foo.5.out <<'_EOF_'
repository/include/my_string.h
work/src/include.h
repository/src/main.c
_EOF_
if test $? -ne 0 ; then fail 5 ; fi

#
pass
