#!/bin/sh
#
#	Create a repository subdirectory.  The repository contains
#	four .c files and a Construct file.  The Construct files
#	specifies Repository {absolute path to repository subdirectory}.
#	Invoke cons in the repository subdirectory.  Check that
#	everything built correctly.  Invoke cons again; check that
#	everything was up-to-date and nothing was built.  Update
#	one .c file in the repository.  Invoke cons in the repository;
#	check that the new .c file was compiled and linked in.
#	Invoke cons again; check that everything was up-to-date
#	and nothing was built.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [multi-module Repository Program within repository]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr, input: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir repository
if test $? -ne 0 ; then exit 2 ; fi

#
cat <<_EOF_ >repository/Construct	# NO SINGLE QUOTES TO EXPAND $work
\$env = new cons;
Repository '$work/repository';
Program \$env 'foo', qw (
	aaa.c
	bbb.c
	ccc.c
	main.c
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/aaa.c
aaa()
{
	printf("repository/aaa.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/bbb.c
bbb()
{
	printf("repository/bbb.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/ccc.c
ccc()
{
	printf("repository/ccc.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/main.c
main()
{
	aaa();
	bbb();
	ccc();
	printf("repository/main.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd repository; $prog .) >cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
diff - cons.1.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
repository/foo > foo.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.out <<'_EOF_'
repository/aaa.c
repository/bbb.c
repository/ccc.c
repository/main.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

(set +x; cd repository; $prog .) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 cons.2.out ; fi
diff - cons.2.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

cat <<'_EOF_' >repository/bbb.c
bbb()
{
	printf("repository/bbb.c #2\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd repository; $prog .) >cons.3.out 2>cons.3.err
if test $? -ne 0 ; then fail 3 cons.3.out ; fi
diff - cons.3.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 3 ; fi
repository/foo > foo.out
if test $? -ne 0 ; then fail 3 ; fi
diff - foo.out <<'_EOF_'
repository/aaa.c
repository/bbb.c #2
repository/ccc.c
repository/main.c
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

(set +x; cd repository; $prog .) >cons.4.out 2>cons.4.err
if test $? -ne 0 ; then fail 4 cons.4.out ; fi
diff - cons.4.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 4 ; fi
diff - cons.4.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 4 ; fi

#
pass
