#!/bin/sh
#
#	In the repository, define a single executable compiled from
#	three .c files in a source subdirectory.  One .c file has
#	a conditional #include for the selected OS.  Compilation
#	takes place in two separate build subdirectories with
#	separate Conscript files that establish 'Link' to the source
#	subdirectory and separate build environments.  Both Conscript
#	builds are invoked from the Construct file.  Build the two
#	executables in the repository.  Invoke cons in the work
#	subdirectory; nothing should get built (repository executables
#	are up-to-date).  Create a work copy of one of the .c files.
#	Invoke cons again in the work subdirectory; check that the
#	executables were constructed correctly and none of the
#	other .o files were created in the work directory.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [multiple-environment variant builds, -R, .c update]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir work/src
if test $? -ne 0 ; then no_result ; fi
mkdir work/build
if test $? -ne 0 ; then no_result ; fi
mkdir work/build/foo
if test $? -ne 0 ; then no_result ; fi
mkdir work/build/bar
if test $? -ne 0 ; then no_result ; fi

mkdir repository
if test $? -ne 0 ; then no_result ; fi
mkdir repository/src
if test $? -ne 0 ; then no_result ; fi
mkdir repository/build
if test $? -ne 0 ; then no_result ; fi
mkdir repository/build/foo
if test $? -ne 0 ; then no_result ; fi
mkdir repository/build/bar
if test $? -ne 0 ; then no_result ; fi

#
cat <<'_EOF_' >repository/Construct
Build qw(
	build/foo/Conscript
	build/bar/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/build/foo/Conscript
Link 'src' => '#src';

$env = new cons(
	CFLAGS	=> '-DFOO',
);

Export qw( env );

Build qw(
	src/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/build/bar/Conscript
Link 'src' => '#src';

$env = new cons(
	CFLAGS	=> '-DBAR',
);

Export qw( env );

Build qw(
	src/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/Conscript
Import qw( env );
Program $env 'xxx', qw (
	a.c
	b.c
	main.c
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/a.c
a()
{
	printf("repository/src/a.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/b.c
b()
{
	printf("repository/src/b.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/main.c
main()
{
	a();
	b();
#ifdef	FOO
	printf("repository/src/main.c:  FOO\n");
#endif
#ifdef	BAR
	printf("repository/src/main.c:  BAR\n");
#endif
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd repository; $prog .) > cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail 1 ; fi
repository/build/foo/src/xxx > foo.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.out <<'_EOF_'
repository/src/a.c
repository/src/b.c
repository/src/main.c:  FOO
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
repository/build/bar/src/xxx > bar.out
if test $? -ne 0 ; then fail 1 ; fi
diff - bar.out <<'_EOF_'
repository/src/a.c
repository/src/b.c
repository/src/main.c:  BAR
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 cons.2.out ; fi
diff - cons.2.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
if test -e work/build/foo/src/a.o ; then fail 2 ; fi
if test -e work/build/foo/src/b.o ; then fail 2 ; fi
if test -e work/build/foo/src/main.o ; then fail 2 ; fi
if test -e work/build/foo/src/xxx ; then fail 2 ; fi
if test -e work/build/bar/src/a.o ; then fail 2 ; fi
if test -e work/build/bar/src/b.o ; then fail 2 ; fi
if test -e work/build/bar/src/main.o ; then fail 2 ; fi
if test -e work/build/bar/src/xxx ; then fail 2 ; fi

cat <<'_EOF_' >work/src/b.c
b()
{
	printf("work/src/b.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.3.out 2>cons.3.err
if test $? -ne 0 ; then fail 3 cons.3.out ; fi
work/build/foo/src/xxx > foo.out
if test $? -ne 0 ; then fail 3 ; fi
diff - foo.out <<'_EOF_'
repository/src/a.c
work/src/b.c
repository/src/main.c:  FOO
_EOF_
if test $? -ne 0 ; then fail 3 ; fi
work/build/bar/src/xxx > bar.out
if test $? -ne 0 ; then fail 3 ; fi
diff - bar.out <<'_EOF_'
repository/src/a.c
work/src/b.c
repository/src/main.c:  BAR
_EOF_
if test $? -ne 0 ; then fail 3 ; fi
if test -e work/build/foo/src/a.o ; then fail 3 ; fi
if test -e work/build/foo/src/main.o ; then fail 3 ; fi
if test -e work/build/bar/src/a.o ; then fail 3 ; fi
if test -e work/build/bar/src/main.o ; then fail 3 ; fi

#
pass
