#!/bin/sh
#
#	In a repository subdirectory, define a single executable
#	compiled from a .c file in source subdirectory, which
#	includes a local .h file.  Both .c and .h have #ifdefs for
#	a selected "OS".  Compilation takes place in two separate
#	build subdirectories established via 'Link' to the source
#	subdirectory.  Each build subdirectory has a Conscript file
#	establishing the build environment.  Build; both copies
#	should be built, each for their "OS".  Build in the work
#	subdirectory; everything should be up-to-date, and no work
#	.o files or executables should be generated.  Create a work
#	copy of the .h file; build; new executables should pick up
#	the new values from the .h file.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [multiple-environment variant builds, -R, .h update]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir work/src
if test $? -ne 0 ; then no_result ; fi
mkdir work/build
if test $? -ne 0 ; then no_result ; fi
mkdir work/build/foo
if test $? -ne 0 ; then no_result ; fi
mkdir work/build/bar
if test $? -ne 0 ; then no_result ; fi

mkdir repository
if test $? -ne 0 ; then no_result ; fi
mkdir repository/src
if test $? -ne 0 ; then no_result ; fi
mkdir repository/build
if test $? -ne 0 ; then no_result ; fi
mkdir repository/build/foo
if test $? -ne 0 ; then no_result ; fi
mkdir repository/build/bar
if test $? -ne 0 ; then no_result ; fi

#
cat <<'_EOF_' >repository/Construct
Build qw(
	build/foo/Conscript
	build/bar/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/build/foo/Conscript
Link 'src' => '#src';

$env = new cons(
	CFLAGS	=> '-DFOO',
	CPPPATH => '.',
);

Export qw( env );

Build qw(
	src/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/build/bar/Conscript
Link 'src' => '#src';

$env = new cons(
	CFLAGS	=> '-DBAR',
	CPPPATH => '.',
);

Export qw( env );

Build qw(
	src/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/Conscript
Import qw( env );
Program $env 'xxx', qw (
	main.c
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/string.h
#ifdef	FOO
#define	H_OS		"FOO"
#endif
#ifdef	BAR
#define	H_OS		"BAR"
#endif
#define	H_STRING	"repository/src/string.h:  %s\n"
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/main.c
#include <src/string.h>
#ifdef	FOO
#define	MAIN_OS		"FOO"
#endif
#ifdef	BAR
#define	MAIN_OS		"BAR"
#endif
main()
{
	printf(H_STRING, H_OS);
	printf("repository/src/main.c:  %s\n", MAIN_OS);
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd repository; $prog .) > cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail 1 ; fi
repository/build/foo/src/xxx > foo.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.out <<'_EOF_'
repository/src/string.h:  FOO
repository/src/main.c:  FOO
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
repository/build/bar/src/xxx > bar.out
if test $? -ne 0 ; then fail 1 ; fi
diff - bar.out <<'_EOF_'
repository/src/string.h:  BAR
repository/src/main.c:  BAR
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 cons.2.out ; fi
diff - cons.2.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
if test -e work/build/foo/src/a.o ; then fail 2 ; fi
if test -e work/build/foo/src/b.o ; then fail 2 ; fi
if test -e work/build/foo/src/main.o ; then fail 2 ; fi
if test -e work/build/foo/src/xxx ; then fail 2 ; fi
if test -e work/build/bar/src/a.o ; then fail 2 ; fi
if test -e work/build/bar/src/b.o ; then fail 2 ; fi
if test -e work/build/bar/src/main.o ; then fail 2 ; fi
if test -e work/build/bar/src/xxx ; then fail 2 ; fi

cat <<'_EOF_' >work/src/string.h
#ifdef	FOO
#define	H_OS		"FOO"
#endif
#ifdef	BAR
#define	H_OS		"BAR"
#endif
#define	H_STRING	"work/src/string.h:  %s\n"
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.3.out 2>cons.3.err
if test $? -ne 0 ; then fail 3 cons.3.out ; fi
work/build/foo/src/xxx > foo.out
if test $? -ne 0 ; then fail 3 ; fi
diff - foo.out <<'_EOF_'
work/src/string.h:  FOO
repository/src/main.c:  FOO
_EOF_
if test $? -ne 0 ; then fail 3 ; fi
work/build/bar/src/xxx > bar.out
if test $? -ne 0 ; then fail 3 ; fi
diff - bar.out <<'_EOF_'
work/src/string.h:  BAR
repository/src/main.c:  BAR
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

#
pass
