#!/bin/sh
#
#	Define an executable to be built from a single .c file in
#	the repository subdirectory.  The .c files prints a single
#	string to be supplied via '-DSTRING=' from CFLAGS in the
#	build environment.  Build the executable in the repository;
#	make sure the executable is build with the default STRING
#	value.  Create a work-subdirectory override file that
#	changes CFLAGS when generating the .o file.  Invoke cons
#	in the work subdirectory with the override file;; make sure
#	the executable is built with the overridden STRING value.
#	Invoke cons again in the work subdirectory, without the
#	override; make sure the executable has reverted to the
#	original STRING value.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [-o overrides, -R]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir repository
if test $? -ne 0 ; then no_result ; fi

#
cat <<'_EOF_' >repository/Construct
$env = new cons (
	COPT	=> '-DSTRING=\"FOO\"',
	CFLAGS	=> '%COPT',
);
Program $env 'foo', 'foo.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/foo.c
main()
{
	printf("repository/foo.c:  %s\n", STRING);
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >work/over
Override '\.o$', COPT => '-DSTRING=\"OVERRIDE\"';
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd repository && $prog .) >cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail 1 ; fi
repository/foo >foo.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.1.out <<'_EOF_'
repository/foo.c:  FOO
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository -o over .) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 ; fi
work/foo >foo.2.out
if test $? -ne 0 ; then fail 2 ; fi
diff - foo.2.out <<'_EOF_'
repository/foo.c:  OVERRIDE
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.3.out 2>cons.3.err
if test $? -ne 0 ; then fail 3 ; fi
work/foo >foo.3.out
if test $? -ne 0 ; then fail 3 ; fi
diff - foo.3.out <<'_EOF_'
repository/foo.c:  FOO
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

#
pass
