#!/bin/sh
#
#	In a repository subdirectory, create Construct and Conscript
#	files for creating four executables, two from
#	.c files in the local directory and two from .c files in a
#	subdirectory.  In the work subdirectory, invoke cons -R to
#	build .o files and executables from these source files.
#	Invoke cons in the repository build repository .o and
#	executable files.  In the work subdirectory, remove one of
#	the local executables and .o file using -r.  Remove the
#	executables and .o files from the subdirectory using -r.
#	Remove the rest of the executables and .o files using -r.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [-r, -R]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir repository
if test $? -ne 0 ; then no_result ; fi
mkdir repository/subdir
if test $? -ne 0 ; then no_result ; fi

#
cat <<'_EOF_' >repository/Construct
$env = new cons;
Program $env 'a', 'a.c';
Program $env 'b', 'b.c';
Build 'subdir/Conscript';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/a.c
main()
{
	printf("repository/a.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/b.c
main()
{
	printf("repository/b.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/subdir/Conscript
$env = new cons;
Program $env 'c', 'c.c';
Program $env 'd', 'd.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/subdir/c.c
main()
{
	printf("repository/subdir/c.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/subdir/d.c
main()
{
	printf("repository/subdir/d.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail 1 ; fi
work/a >a.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - a.1.out <<'_EOF_'
repository/a.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
work/b >b.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - b.1.out <<'_EOF_'
repository/b.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
work/subdir/c >c.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - c.1.out <<'_EOF_'
repository/subdir/c.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
work/subdir/d >d.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - d.1.out <<'_EOF_'
repository/subdir/d.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

# Make the repository writable again so we can build in it
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u+w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd repository && $prog .) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 ; fi
repository/a >a.2.out
if test $? -ne 0 ; then fail 2 ; fi
diff - a.2.out <<'_EOF_'
repository/a.c
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
repository/b >b.2.out
if test $? -ne 0 ; then fail 2 ; fi
diff - b.2.out <<'_EOF_'
repository/b.c
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
repository/subdir/c >c.2.out
if test $? -ne 0 ; then fail 2 ; fi
diff - c.2.out <<'_EOF_'
repository/subdir/c.c
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
repository/subdir/d >d.2.out
if test $? -ne 0 ; then fail 2 ; fi
diff - d.2.out <<'_EOF_'
repository/subdir/d.c
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

# Make the repository non-writable again,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository -r b.o b) >cons.3.out 2>cons.3.err
if test $? -ne 0 ; then fail 3 cons.3.out ; fi
work/a >a.3.out
if test $? -ne 0 ; then fail 3 ; fi
diff - a.3.out <<'_EOF_'
repository/a.c
_EOF_
if test $? -ne 0 ; then fail 3 ; fi
if test -e work/b.o ; then fail 3 ; fi
if test -e work/b ; then fail 3 ; fi
work/subdir/c >c.3.out
if test $? -ne 0 ; then fail 3 ; fi
diff - c.3.out <<'_EOF_'
repository/subdir/c.c
_EOF_
if test $? -ne 0 ; then fail 3 ; fi
work/subdir/d >d.3.out
if test $? -ne 0 ; then fail 3 ; fi
diff - d.3.out <<'_EOF_'
repository/subdir/d.c
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

(set +x; cd work && $prog -R $work/repository -r subdir) >cons.4.out 2>cons.4.err
if test $? -ne 0 ; then fail 4 cons.4.out ; fi
work/a >a.4.out
if test $? -ne 0 ; then fail 4 ; fi
diff - a.4.out <<'_EOF_'
repository/a.c
_EOF_
if test $? -ne 0 ; then fail 4 ; fi
if test -e work/b.o ; then fail 4 ; fi
if test -e work/b ; then fail 4 ; fi
if test -e work/subdir/c.o ; then fail 4 ; fi
if test -e work/subdir/c ; then fail 4 ; fi
if test -e work/subdir/d.o ; then fail 4 ; fi
if test -e work/subdir/d ; then fail 4 ; fi

(set +x; cd work && $prog -R $work/repository -r .) >cons.5.out 2>cons.5.err
if test $? -ne 0 ; then fail 5 cons.5.out ; fi
if test -e work/a.o ; then fail 5 ; fi
if test -e work/a ; then fail 5 ; fi
if test -e work/b.o ; then fail 5 ; fi
if test -e work/b ; then fail 5 ; fi
if test -e work/subdir/c.o ; then fail 5 ; fi
if test -e work/subdir/c ; then fail 5 ; fi
if test -e work/subdir/d.o ; then fail 5 ; fi
if test -e work/subdir/d ; then fail 5 ; fi

#
pass
